@@ -117,6 +117,9 @@ static struct array *array_new() {
 	struct array *tmp;
 
 	tmp = (struct array *)malloc(sizeof(*tmp));
+	if (!tmp)
+		rl_fatal(EX_SOFTWARE, "ABORT - Can't allocate memory");
+
 	memset(tmp, 0, sizeof(*tmp));
 #ifdef MEMDEBUG
 	tmp->id = strdup("array");
@@ -143,6 +146,8 @@ static void array_append(struct array *arr, rl_opcode_t op) {
 	int i = arr->len;
 	
 	arr->elems = realloc(arr->elems, ++arr->len * sizeof(rl_opcode_t));
+	if (!arr->elems)
+		rl_fatal(EX_SOFTWARE, "ABORT - Can't allocate memory");
 	arr->elems[i] = op;
 }
 
@@ -153,8 +158,12 @@ static struct array *array_copy(struct array *a) {
 		return NULL;
 	tmp = array_new();
 	tmp->len = a->len;
-	if(a->len)
+	if(a->len) {
 		tmp->elems = (rl_opcode_t *)malloc(sizeof(rl_opcode_t) * a->len);
+		if (!tmp->elems)
+			rl_fatal(EX_SOFTWARE, "ABORT - Can't allocate memory");
+	}
+
 	memcpy(tmp->elems, a->elems, sizeof(rl_opcode_t) * a->len);
 	return tmp;
 }
@@ -194,11 +203,16 @@ static struct opmeta *opmeta_new(int len) {
 	struct opmeta *tmp;
 
 	tmp = (struct opmeta *)malloc(sizeof(*tmp));
+	if (!tmp)	
+		rl_fatal(EX_SOFTWARE, "ABORT - Can't allocate memory");
 	memset(tmp, 0, sizeof(*tmp));
 	tmp->after = array_new();
 	tmp->ops = array_new();
-	if(len)
+	if(len) {
 		tmp->fixup = (rl_opcode_t **)malloc(len * sizeof(rl_opcode_t *));
+		if (!tmp->fixup)
+			rl_fatal(EX_SOFTWARE, "ABORT - Can't allocate memory");
+	}
 	memset(tmp->fixup, 0, len * sizeof(rl_opcode_t *));
 #ifdef MEMDEBUG
 	tmp->id = strdup("opmeta");
@@ -254,6 +268,8 @@ struct opmeta *opmeta_make(int len, rl_opcode_t op, ...) {
 		opm->after = tmp;
 		opm->bytes = (rl_opcode_t *)
 			realloc(opm->bytes, sizeof(rl_opcode_t) * (1 + i + opm->len));
+		if (!opm->bytes)
+			rl_fatal(EX_SOFTWARE, "ABORT - Can't allocate memory");
 		array_append(opm->ops, op);
 		do {
 			opm->bytes[opm->len++] = op;
@@ -281,6 +297,8 @@ struct opmetalist *opmetalist_new() {
 	struct opmetalist *opml;
 	
 	opml = (struct opmetalist *)malloc(sizeof(*opml));
+	if (!opml)
+		rl_fatal(EX_SOFTWARE, "ABORT - Can't allocate memory");
 	memset(opml, 0, sizeof(*opml));
 #ifdef MEMDEBUG
 	opml->id = strdup("opmetalist");
@@ -303,6 +321,8 @@ int opmetalist_add(struct opmetalist *opml, struct opmeta *opm) {
 		if(array_intersect_p(opm->ops, opml->opms[i++]->after))
 			return 1;
 	opml->opms = realloc(opml->opms, ++opml->len * sizeof(*opml->opms));
+	if (!opml->opms)
+		rl_fatal(EX_SOFTWARE, "ABORT - Can't allocate memory");
 	memmove(opml->opms + index + 1, opml->opms + index,
 					sizeof(*opml->opms) * (opml->len - index - 1));
 	opml->opms[index] = opm;
@@ -344,13 +364,18 @@ struct oplist *opmetalist_resolve(struct opmetalist *o) {
 	struct oplist *ret;
 
 	ret = (struct oplist *)malloc(sizeof(*ret));
+	if (!ret)
+		rl_fatal(EX_SOFTWARE, "ABORT - Can't allocate memory");
 	ret->ops_len = 0;
 	for(i = 0; i < o->len; i++) {
 		opmeta_resolve(o->opms[i]);
 		ret->ops_len += o->opms[i]->len;
 	}
-	if(ret->ops_len)
+	if(ret->ops_len) {
 		ret->ops_list = (rl_opcode_t *)malloc(sizeof(*ret->ops_list) * ret->ops_len);
+		if (!ret->ops_list)
+			rl_fatal(EX_SOFTWARE, "ABORT - Can't allocate memory");
+	}
 	k = 0;
 	for(i = 0; i < o->len; i++)
 		for(j = 0; j < o->opms[i]->len; j++) {
@@ -361,3 +386,5 @@ struct oplist *opmetalist_resolve(struct opmetalist *o) {
 		}			
 	return ret;
 }
+
+/* vim: set ts=2: */
