@@ -32,6 +32,8 @@ struct rl_stack *rlstk_new() {
 	struct rl_stack *s;
 
 	s = malloc(sizeof(*s));
+	if (!s)
+		rl_fatal(EX_SOFTWARE, "ABORT - Can't allocate memory");
 	memset(s, 0, sizeof(*s));
 	return s;
 }
@@ -39,3 +41,5 @@ struct rl_stack *rlstk_new() {
 void rlstk_free(struct rl_stack *s) {
 	free(s);
 }
+
+/* vim: set ts=2: */
